@@ -36,8 +36,8 @@ Package::Package(PackageStructure *structure)
     d->structure = structure;
 
     if (d->structure) {
-        d->structure.data()->initPackage(this);
         addFileDefinition("metadata", QStringLiteral("metadata.json"));
+        d->structure.data()->initPackage(this);
     }
 }
 
@@ -59,6 +59,7 @@ Package &Package::operator=(const Package &rhs)
 
 bool Package::hasValidStructure() const
 {
+    qWarning() << d->structure << requiredFiles();
     return d->structure;
 }
 
@@ -179,7 +180,7 @@ KPluginMetaData Package::metadata() const
 {
     // qCDebug(KPACKAGE_LOG) << "metadata: " << d->path << filePath("metadata");
     if (!d->metadata && !d->path.isEmpty()) {
-        const QString metadataPath = filePath("metadata");
+        const QString metadataPath = filePath("metadata", QStringLiteral("metadata.json"));
 
         if (!metadataPath.isEmpty()) {
             d->createPackageMetadata(metadataPath);
@@ -290,7 +291,7 @@ bool PackagePrivate::isInsidePackageDir(const QString &canonicalPath) const
 
 QString Package::filePath(const QByteArray &fileType, const QString &filename) const
 {
-    if (!d->valid) {
+    if (!d->valid && d->checkedValid) { // Don't check the validity here, because we'd have infinite recursion
         QString result = d->fallbackFilePath(fileType, filename);
         if (result.isEmpty()) {
             // qCDebug(KPACKAGE_LOG) << fileType << "file with name" << filename
@@ -540,7 +541,11 @@ void Package::setPath(const QString &path)
 
     // .. but something did change, so we get rid of our discovery cache
     d->discoveries.clear();
-    d->metadata = std::nullopt;
+
+    // Do NOT override the metadata when the PackageStructure has set it
+    if (!previousPath.isEmpty()) {
+        d->metadata = std::nullopt;
+    }
 
     // uh-oh, but we didn't end up with anything valid, so we sadly reset ourselves
     // to futility.
